<?php


namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
abstract class ChartViewAbstract
{
    const DATASOURCE_ID_KEY = "_id";
    
	public function fetchWidgetData(){
        $widget = $this->widget;
        $parameters = $widget['parameters'];

        $postGroupConditions = !empty($widget["postGroupConditions"]) ? $widget["postGroupConditions"] : [];
        $fullAccessProfiles = !empty($parameters['fullAccessProfiles']) ? $parameters['fullAccessProfiles'] : [];
        $previousConditions = !empty($parameters["previousConditions"]) ? $parameters["previousConditions"] : [];
        $postConditions = !empty($parameters["postConditions"]) ? $parameters["postConditions"] : [];

        if(!empty($parameters["dimensions"])){
            $dimensions = $parameters["dimensions"];
        } else {
            $dimensions = !empty($parameters['dimensionName']) ? $parameters['dimensionName'] : "";
        }
        $limitRows = !empty($parameters["limitRows"]) ? $parameters["limitRows"] : null;
        $orderBy = [];
        if (!empty($parameters["orderByName"]) && !empty($parameters["orderByDirection"])) {
            $orderBy = [$parameters["orderByName"] => $parameters["orderByDirection"]];
        }
        /** @var \Service\WidgetData $widgetDataService */
        $widgetDataService = InstanceManager::getInstance()->getService('\Service\WidgetData');
        $this->widgetData = $widgetDataService->fetchWidgetData(
                $widget['_id'], $parameters['structureId'], null, $parameters['period'],
				$parameters['facts'],
                $dimensions,
                $orderBy,
				null,$limitRows,$parameters['drillDownFilters'], false,
                $previousConditions, $postConditions, $postGroupConditions,
                null,$widget,
                null, null,
                null, null,
                false, $fullAccessProfiles);
    }

    public function dataSourceAddIndex($dataSource){
        $newDataSource = [];
        foreach($dataSource as $index=>$data){
            $data[self::DATASOURCE_ID_KEY] = $index;
            array_push($newDataSource, $data);
        }
        return $newDataSource;
    }

    public function getUserViews(){
        $viewService = InstanceManager::getInstance()->getService('\Service\View');
        $userId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
        return $viewService->getUserViews($userId);
    }

    public function getWidgetUserView($widget, $dashboardId){
        $userViews = $this->getUserViews();
        $widgetId = $widget["_id"];
        if (!empty($userViews[$dashboardId])) {
            $userViews = $userViews[$dashboardId];
            if (!empty($userViews[$widgetId])){
                return $userViews[$widgetId];
            }else if(!empty($userViews["hidden"][$widgetId])){
                return $userViews["hidden"][$widgetId];
            }
        }
        return null;
    }

    protected function getCardValueFontSize($max = 100){
        $userView = $this->getWidgetUserView($this->widget, $this->dashboardId);
        $width = 18;
        if (!empty($userView)){
            $width = !empty($userView["w"]) ? $userView["w"] : 18;
        }
        return $width / 9 * $max;
    }
}