<?php
namespace Zmart\Mongo;

class Mongo extends \Zeedhi\Framework\DB\Mongo\Mongo {

    protected $dbNamePrefix;
    private $mongoOptions = [];
    private $port;
    private $server;


    public function __construct($server, $port, $dbNamePrefix, $mongoOptions) {
        if( $mongoOptions == null ){
            $mongoOptions = [];
        }
        $mongoOptions['typeMap'] = [
            'array' => 'array',
            'document' => 'array',
            'root' => 'array',
        ];
       // $mongoOptions['maxTimeMS'] = 1800000;
        $this->mongoOptions = $mongoOptions;
        $this->port = $port;
        $this->server = $server;
        $this->dbNamePrefix = $dbNamePrefix;
        parent::__construct($server, $port, null, $mongoOptions);
    }

    public function setDbByNameBase($dbName) {
        if($dbName){
            $this->mongo = new \MongoDB\Client("mongodb://$this->server:$this->port",[], $this->mongoOptions);
            $this->db = $this->mongo->selectDatabase($dbName);
        }
    }

    public function setDbByName(string $dbName = null) {
        if($dbName){
            $dbName = $this->dbNamePrefix.$dbName;
            $this->mongo = new \MongoDB\Client("mongodb://$this->server:$this->port",[], $this->mongoOptions);
            $this->db = $this->mongo->selectDatabase($dbName);
        }
    }

    /**
     * Find in database for documents that match with given criteria.
     *
     * @param string $collectionName
     * @param array $criteria
     * @param array $sort
     * @param int $limit
     *
     * @param $numSkip
     * @return array
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    public function findPagination($collectionName, $criteria = array(), $sort = array(), $limit = 0, $numSkip = 0) {
        $this->validateDbSetted();
        $collection = $this->getCollection($collectionName);

        $option = [];
        $option["limit"] = $limit;
        $option["sort"] = $sort;
        $option["skip"] = $numSkip;
        $cursor = $collection->find($criteria,$option);
        $result = [];
        foreach($cursor as $document){
            if(is_object($document)){
                $result[] = get_object_vars($document);
            }else{
                $result[] = $document;
            }
        }
        return $result;
    }

    /**
     * @param $collectionName
     * @param array $criteria
     * @return int
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     */
    public function count($collectionName, $criteria = array()) {
        $this->validateDbSetted();
        $collection = $this->getCollection($collectionName);
        return $collection->count($criteria);
    }

    /**
     * Find in database for documents that match with given criteria.
     *
     * @param string $collectionName
     * @param array $criteria
     * @param array $criteria
     * @param array $sort
     * @param int $limit
     *
     * @return array
     * @throws \Zeedhi\Framework\DB\Mongo\Exception
     *
     * @throws Exception When db name is not setted.
     */
    public function find($collectionName, $criteria = array(), $sort = null, $limit = null) {
        $this->validateDbSetted();
        $collection = $this->getCollection($collectionName);
        $option = [];
        $option["limit"] = $limit;
        $option["sort"] = $sort;
        $cursor = $collection->find($criteria,$option);
        $result = [];
        foreach($cursor as $document){
            if(is_object($document)){
                $result[] = get_object_vars($document);
            }else{
                $result[] = $document;
            }
        }
        return $result;
    }

    /**
     * Defines condition before find.
     *
     * Aggregate conditions to Mongo::find, using aggregators it simulates the SQL where clause.
     *
     * @param string $collectionName
     * @param array $criteria
     *
     * @return array
     * @throws Exception
     */
    public function aggregate(string $collectionName, array $criteria) : array {
        $this->validateDbSetted();
        $collection = $this->getCollection($collectionName);
        try {
            $cursor = $collection->aggregate($criteria);
            $result = [];
            foreach($cursor as $document){
                if(is_object($document)){
                    $result[] = get_object_vars($document);
                }else{
                    $result[] = $document;
                }
            }
            return $result;
        } catch (\MongoDB\Driver\Exception\Exception $e) {
            throw Exception::aggregateError($e);
        }
    }

    public function command(array $command) : array {
        $this->validateDbSetted();
        $cursor = $this->db->command($command);
        $result = [];

        foreach($cursor as $document){
            if(is_object($document)){
                $result[] = get_object_vars($document);
            }else{
                $result[] = $document;
            }
        }
        return $result;
    }

    public function update(string $collectionName, array $criteria, $update, bool $upsert = false, bool $multi = false): \MongoDB\UpdateResult
    {
        $this->validateDbSetted();
        $options = array(
            "upsert" => $upsert
        );
        if ($multi) {
            $result = $this->getCollection($collectionName)->updateMany($criteria, $update, $options);
        }else{
            $result = $this->getCollection($collectionName)->replaceOne($criteria, $update['$set'], $options);


        }
        return $result;
    }

}